# -*- coding: utf-8 -*
import time
from pinpong.board import gboard, I2C

I2C_ADDR = 0x40  # i2c address
INQUIRYSTATUS = 0x21  # Check status
ENTERSAVEELETRI = 0x88  #
WAKEUP = 0xFF  # Wake-up command

START_SYNTHESIS = 0x01  # start synthesis command 0
START_SYNTHESIS1 = 0x05  # Start synthesis command 1
STOP_SYNTHESIS = 0x02  # End speech synthesis
PAUSE_SYNTHESIS = 0x03  # pause speech synthesis command
RECOVER_SYNTHESIS = 0x04  # Resume speech synthesis commands
I2C_MODE = 0x01
UART_MODE = 0x02

MALE = 0x04  # Male voice
FEMALE = 0x03  # female voice
DONALDDUCK = 0x05  # Voice of Donald Duck

ALPHABET = 0X06  # Spell
WORD = 0X07  # word


CHINESE = 0XA0  # Chinese
ENGLISH = 0XA1  # English
NONE = 0XA2   #
CATON = 0XA3  # Word by word
SMOOTH = 0XA4  # Fluently
PINYIN_ENABLE = 0XA5  # Use Pinyin to read
PINYIN_DISABLE = 0XA6  # don't use pinyin pronunciation
CHINESEL = 0XA7   #
ENGLISHL = 0XA8   #
AUTOJUDGEL = 0XA9  # Auto Judge
NUMBER = 0XAA  # Telephone number
NUMERIC = 0XAB   #
AUTOJUDGED = 0XAC  # Auto Judge
ZREO = 0XAD  # Read as 'zero'
OU = 0XAE  # Read as'ou'
YAO = 0XAF  # Read as 'yao'
CHONE = 0XB0  # Read as 'yi'
NAME = 0XB1   #
AUTOJUDGEDN = 0XB2   #
ERROR = -1

class DFRobot_SpeechSynthesis(object):
    V1 = 1
    V2 = 2

    def __init__(self):
        self._is_V1 = False
        self._index = 0
        self._len = 0

    def begin(self, version):
        init = [0xAA]
        if version == self.V1:
            print("TTS V1 begin starts...")
            self._is_V1 = True
            self.write_cmd(init)
            time.sleep(0.05)
            self.speak_lish("[n1]")
            self.set_voice(5)
            self.set_speed(5)
            self.set_tone(5)
            self.set_soundType(FEMALE)
            self.set_english_pron(WORD)
            print("TTS V1 begin end")
        else:
            print("TTS V2 begin starts...")
            for i in range(40):
                self.write_cmd(init)
                time.sleep(0.05)
                check = [0xFD, 0x00, 0x01, 0x21]
                self.write_cmd(check)
                if self.read_ack(1)[0] == 0x4F:
                    print("TTS V2 begin ok!")
                    break
                else:
                    print("TTS V2 begin retry:"+str(i))
            print("TTS V2 begin end")
            self.set_voice(1)

    def set_voice(self, voc):
        str = "[v3]"
        if voc > 9:
            voc = 9
        str = str[:2] + chr(48 + voc) + str[3:]
        self.speak_lish(str)

    def speak(self, string):
        str = string.encode('gb2312')
        head = [0xfd, 0x00, 0x00, 0x01, 0x00]
        data = list(str)
        data2 = (list(data))
        lenght = len(data2) + 2
        head[1] = lenght >> 8
        head[2] = lenght & 0xff
        data1 = head + data2
        # self.read_ack(1)
        # self.read_ack(1)
        self.write_cmd(data1)
        self.wait()
        return

    def wait(self):
        if self._is_V1:
            while True:
                if self.readACK() == 0x41:
                    break
                time.sleep(0.2)
            while True:
                if self.readACK() == 0x4F:
                    break
                time.sleep(0.2)
        else:
            while True:
                if self.readACK() == 0x41:
                    break
                time.sleep(0.2)
            time.sleep(0.1)
            while True:
                check = [0xFD, 0x00, 0x01, 0x21]
                self.write_cmd(check)
                if self.readACK() == 0x4f:
                    break
                time.sleep(0.02)
                
    def readACK(self):
        data = self.read_ack(1)
        return data[0]

    def speak_lish(self, word):
        self._len = len(word)
        point = 0
        _unicode = [0 for i in range(self._len)]
        while self._index < self._len:
            _unicode[point] = ord((word[self._index])) & 0x7f
            self._index += 1
            point += 1
        self.send_pack(START_SYNTHESIS1, _unicode, self._len)
        self.wait()
        self._index = 0
        self._len = 0

    def send_pack(self, cmd, data, len):
        length = 0
        head = [0, 0, 0, 0, 0]
        head[0] = 0xfd
        if cmd == START_SYNTHESIS:
            pass
        elif cmd == START_SYNTHESIS1:
            length = 2 + len
            head[1] = length >> 8
            head[2] = length & 0xff
            head[3] = START_SYNTHESIS
            if self._is_V1:
                head[4] = 0x00
            else:
                head[4] = 0x04
            self.send_command(head, data, len)

    def send_command(self, head, data, len):
        self.write_cmd(head)
        self.write_cmd(data)

    def set_speed(self, speed):
        str = "[s5]"
        if speed > 9:
            speed = 9
        str = str[:2] + chr(48 + speed) + str[3:]
        self.speak_lish(str)

    def set_soundType(self, type):
        if(type == MALE):
            str = "[m51]"
            self.speak_lish(str)
        elif(type == FEMALE):
            str = "[m3]"
            self.speak_lish(str)
        elif(type == DONALDDUCK):
            str = "[m54]"
            self.speak_lish(str)
        else:
            print("no that type")


    def set_sound_type(self, type):
        if(type == MALE):
            str = "[m51]"
            self.speak_lish(str)
        elif(type == FEMALE):
            str = "[m3]"
            self.speak_lish(str)
        elif(type == DONALDDUCK):
            str = "[m54]"
            self.speak_lish(str)
        else:
            print("no that type")

    def set_tone(self, tone):
        str = "[t5]"
        if tone > 9:
            tone = 9
        str = str[:2] + chr(48 + tone) + str[3:]
        self.speak_lish(str)

    def set_english_pron(self, pron):
        if(pron == ALPHABET):
            str = "[h1]"
            self.speak_lish(str)
        elif(pron == WORD):
            str = "[h2]"
            self.speak_lish(str)

    def enable_rhythm(self, enable):
        if(enable == True):
            str1 = "[z1]"
        elif(enable == False):
            str1 = "[z0]"
        self.speak_lish(str1)

    def set_digital_pron(self, pron):
        if(pron == NUMBER):
            str1 = "[n1]"
        elif(pron == NUMERIC):
            str1 = "[n2]"
        elif(pron == AUTOJUDGED):
            str1 = "[n0]"
        self.speak_lish(str1)

    def enable_pinyin(self, enable):
        if(enable == True):
            str1 = "[i1]"
        elif(enable == False):
            str1 = "[i0]"
        self.speak_lish(str1)

    def set_speech_style(self, style):
        if(style == CATON):
            str1 = "[f0]"
        elif(style == SMOOTH):
            str1 = "[f1]"
        else:
            str1 = "[f0]"
        self.speak_lish(str1)

    def set_language(self, style):
        if(style == CHINESEL):
            str1 = "[g1]"
        elif(style == ENGLISHL):
            str1 = "[g2]"
        elif(style == AUTOJUDGEL):
            str1 = "[g0]"
        else:
            str1 = "[g1]"
        self.speak_lish(str1)

    def set_zero_pron(self, pron):
        if(pron == ZREO):
            str1 = "[o0]"
        elif(pron == OU):
            str1 = "[o1]"
        else:
            str1 = "[o0]"
        self.speak_lish(str1)

    def set_one_pron(self, pron):
        if(pron == YAO):
            str1 = "[y0]"
        elif(pron == CHONE):
            str1 = "[y1]"
        else:
            str1 = "[y0]"
        self.speak_lish(str1)

    def set_name_pron(self, pron):
        if(pron == NAME):
            str1 = "[r1]"
        elif(pron == AUTOJUDGEDN):
            str1 = "[r0]"
        else:
            str1 = "[r1]"
        self.speak_lish(str1)

    def reset(self):
        self.speak_lish("[d]")

class DFRobot_SpeechSynthesis_I2C(DFRobot_SpeechSynthesis):
    def __init__(self, board=None, i2c_addr=0x40, bus_num=0):
        if isinstance(board, int):
            i2c_addr = board
            board = gboard
        elif board is None:
            board = gboard
        self.__addr = i2c_addr
        try:
            self._i2c = I2C(bus_num)
        except Exception as e:
            print(f"\nError:PinPong未初始化")
            print(f"Error:{e}\n")
            raise
        super(DFRobot_SpeechSynthesis_I2C, self).__init__()

    def write_cmd(self, data):
        self._i2c.writeto(self.__addr, data)

    def read_ack(self, len):
        rslt = self._i2c.readfrom(self.__addr, len)
        if rslt is None:
            print(f"TTS rslt:{rslt}")
            return ([0]*len)
        else:
            return rslt

